#!/bin/sh 

# Usage: ./scan9000.sh -a [-d non-depot application path]
#        ./scan9000.sh [-d non-depot application path] [-p depot application]
# +----------------------------------------------------------------+
# |                                                                |
# | This script will analyze an HP9000 PA-RISC system, and output  |
# | an HTML report, listing which products can (and cannot) be     |
# | ported to an HP-UX Integrity (Itanium) system running HP9000   |
# | Containers.                                                    |
# |                                                                | 
# | Copyright (c) 2011 Radiatus 2011.  All rights reserved.        |
# |                                                                |
# +----------------------------------------------------------------+

# script version
scan9k_version=1.0.2

# Find the script name and script directory 
script_name=`basename $0`
script_path=`dirname $0`
cmd_line="./$script_name $*";

# Change to script directory and then start processing
cd "$script_path";

lock_file="/tmp/mg_checker.lock";
start_date=`date`;
TTY=`tty`;

tmp_dir=`mktemp -d $PWD`;
report_file="Scan9000result.html";
output_html_file="${tmp_dir}/${report_file}";

host_name=`hostname`;
sys_model=`model`;
sys_os=`uname -svr`;

############################ Common functions ##########################

# print_err <string>
# Print an error message, and then exit the script.
function print_err {
   echo "ERROR: $*" 1>&2
   exit 1;
}

# Print a message to standard out
function print_stdout {
   echo "$*\c" > $TTY
}

# Check for the following prerequisites:
# - The system must be an HP-9000 (PA-RISC) system.
# - The user must be running as root
# - Check some basic commands are available and usable

function check_prereq {
    username=`id -nu`;
    uid=`id -u`;
    release=`uname -r`;
    platform=`uname -m`;

    echo "$platform" | grep 9000 > /dev/null
    if [ $? != 0 ]
    then
        print_err "This is not an HP9000 server. The uname -m returned $platform";
    fi
    if [ "$uid" -ne 0 ]; then
        print_err "You are not a privileged user"
    fi

    OLDIFS=$IFS
    while IFS='=' read my_cmd my_cmd_access; do
        $my_cmd_access > /dev/null 2>&1
        if [ "$?" != "0" ]; then
            print_err "The command $my_cmd is either not found or un-usable";
        fi
    done <<CMD_END
strings=echo testing | strings
swlist=swlist
CMD_END
    IFS=$OLDIFS
}

function print_usage {
   cat <<EOF_USAGE
Usage: ./$script_name OPTIONS

OPTIONS:
  -a	    Scan all the depot products on the system.

	    ./$script_name -a

  -p name   Scan selected depot products only.Pass either a bundle
	    or product name.This -p option can be used multiple times.
	    It cannot be used with the -a option.
	    ./$script_name -p product1 -p product2

            ./$script_name -p openssl -p hpuxwsApache

  -d dir    Scan application directories.This is intended for products that
            are not installed as depots.The -d option can be used multiple times,
            and can be used with either the -a or -p options.
            ./$script_name [-a -d directory1 -d directory2] -p product1 -p product2

            ./$script_name -d /usr/local/tcpdump -d /usr/local/hypermail
            ./$script_name -a -d /usr/local/tcpdump -d /usr/local/hypermail
            ./$script_name -p openssl -d /usr/local/tcpdump -d /usr/local/hypermail

  -h	    display this help and exit

EOF_USAGE
    #trap - 0
    exit 1
}

function startup_msg {
    echo "Starting Analysis script at: $start_date"
    echo "Temporary directory path is: $tmp_dir\n"
}

# The cleanup function is called when the script is interrupted.
# Remove all temporary files, and the lock file.
function cleanup {
    echo "\nCleaning up temporary files \c";
    rm -rf $tmp_dir;
    rm -f $lock_file;
    echo " ... done.                    \n"
}

# Create a html report file.
function create_result {
    echo "\nCreating the result file \c";
    date_format=`date +%Y"_"%B%d"_"%H%M%S `;
    bare_name=`echo $report_file | sed 's/\.html//'`;
    cp $output_html_file $report_file;
    cp "$output_html_file" "${bare_name}_${date_format}.html";
    echo "... done.";
    echo "The report is available at: $PWD/$report_file";
}

# does_program_use_device_files(string filename, string reason_file)
#
# Check if the given filename uses any device files (like /dev/escsi).
# Exclude devices /dev/null, /dev/zero, /dev/random, and /dev/urandom.
# If yes, write the filename, and list of devices, to reasonfile.

function does_program_use_device_files {
    filename=$1
    reason_file=$2
    devices=`strings $filename | grep "^/dev/" | grep -v -E\
        "/dev/null|/dev/zero|/dev/random|/dev/urandom|\
        /dev/srandom|/dev/egd-pool|/dev/console|/dev/tty" | sort | uniq`
    if [ "$devices" != "" ]; then
        echo "The program $filename uses the following device files: <b>$devices</b> <p>" >> $reason_file
        echo "" >> $reason_file
    fi
}

# does_program_use_unsupported_commands(string filename, string reason_file)
#
# Check if the given filename uses any unsupported commands.
# If yes, write the filename, and unsupported commands, to reason_file.

function does_program_use_unsupported_commands {
    filename=$1
    reason_file=$2
    commands=`strings $filename | grep -F -f unsupported_commands.txt | sort | uniq`;
    if [ "$commands" != "" ]; then
        echo "The program $filename uses the following unsupported commands: <b>$commands</b><p>" >> $reason_file
        echo " " >> $reason_file
    fi
}

# int check_if_known_product(string product, string migrate_reason_file)
#
# The table 'known_product_migration.txt' the 'can migrate' status (yes/no)
# and migration details, for a set of known products.
#
# Check if the given product is in the known_product_migration.txt table.
# If yes, update the reasonfile with the migration details, and return 0.
# If not, return 1
function check_if_known_product {
    product=$1
    reasonfile=$2
    entry=`grep "^$product " known_product_migration.txt`
    if [ "$entry" != "" ];
    then
        echo "$entry" | grep "| no |" > /dev/null
        if [ $? = 0 ]; then
           reason=`echo $entry | awk -F "|" '{print $4}'`
           echo "$reason" >> $reasonfile
        fi
        return 0
    fi
    return 1
}

# int check_if_kernel_module(string product, string file_list, string reasonfile)
#
# The file_list contains the output of "swlist -l product -l file", which is:
#   <product>.<fileset>: <filename>
#   openssl.OPENSSL-RUN: /opt/openssl/0.9.7/bin/openssl
#
# Check if the given product contains any kernel modules (/usr/conf/lib, /usr/conf/mod).
# If true, update the reasonfile with the details, and return 0.
# If not, return 1

function check_if_kernel_module {
    product=$1
    file_list=$2
    reasonfile=$3

    kernel_files=`cat $file_list | grep "  $product\." | grep -E "/usr/conf/lib|/usr/conf/mod" |\
		 awk '{print $2}'`
    if [ "$kernel_files" != "" ];
    then
       echo "The product contains the following kernel files:<p>" >> $reasonfile
       echo "$kernel_files" >> $reasonfile
       return 0
    fi
    return 1
}

# void check_product_files(string product, string file_list, string reason_file)
#
# The file_list contains the output of "swlist -l product -l file", which is:
#   <product>.<fileset>: <filename>
#   openssl.OPENSSL-RUN: /opt/openssl/0.9.7/bin/openssl
#
# Use the file_list to retrieve the files contained in the product.
# For each executable file in the product:
# - Check if it opens a device file (/dev)
# - Check if it uses an unsupported command (that is, unsupported under HP-9000 conatiners)
# Update the reasonfile with the details

function check_product_files {
    product=$1
    file_list=$2
    reason_file=$3

    files=`cat $file_list | grep "  $product\." | awk '{print $2}'`
    for single_file in `echo $files`; do
        is_pa_risc=`file $single_file | grep "PA-RISC"`
        is_text=`file $single_file | grep "commands text"`
        is_shell_script=""
        if [ "$is_text" != "" ]; then
            is_shell_script=`head -n 1 $single_file | grep "#\!"`
        fi
        if [ "$is_pa_risc" != "" -o "$is_shell_script" != "" ]; then
            does_program_use_device_files $single_file $tmp_dir/migrate_problems.txt
            does_program_use_unsupported_commands $single_file $tmp_dir/migrate_problems.txt
        fi
    done
}

# write_html_application_row(string product, string reasonfile, string dependency_file, 
#                            string reason_id, string dependency_id)
#
# Write an HTML table row to $output_html_file containing
# - The product name, version, description
# - Whether or not it can be migrated
# - The detailed reasons why it cannot be migrated 
# - The list of dependencies the product has
#
# The 'reason_file' contains the detailed reason why the product cannot be migrated.
# If the reason_file is file size 0, then the product can be migrated.
#
# The reason_id and dependency_id are used has HTML tag ids, in order to easily
# show/hide the reason details.

function write_html_application_row {
    app_name=$1;
    file_reason=$2;
    file_dependencies=$3;
    file_size=$( ls -l $file_reason | awk '{print $5}' )
    my_id=$4;
    my_id2=$5;

    # If the 'reasons' file is empty, then there are no migration problems
    if [ "$file_size" = "0" ]; then
        status='Yes';
	td_class="green";
	id_name="migrate";
	id_val="yes";
    else
        status='Issue';
	td_class="red";
	id_name="cannotmig";
	id_val="no"
    fi

    # Get the description of the product
    if [ ! -e $tmp_dir/sd_description.txt ]; then
        swlist -l product | grep -v "^#" | grep -v "^#" > $tmp_dir/sd_description.txt
    fi 

    app_details[]=`awk '{ if($1=="'$app_name'") {print $2;$1="";$2="";print}}' $tmp_dir/sd_description.txt`;
    set -A RES $app_details
    version=${RES[0]};
    if [ "$version" = "" ]; then
        version="---"
    fi
    set +A RES ""
    description=${RES[*]};
    if [ "$description" = "" ]; then
        description="---";
    fi
    reason=`cat $file_reason 2> /dev/null`;
    if [ "$reason" = "" ]; then
        reason="---"
    fi
    dependencies=`cat $file_dependencies 2> /dev/null`;
        echo "<tr $id_name=$id_val>" >> $output_html_file
        echo "<td>$app_name</td>" >> $output_html_file
        echo "<td>$version</td>" >> $output_html_file
        echo "<td>$description</td>" >> $output_html_file
        echo "<td class="$td_class">$status</td>" >> $output_html_file
cat <<End >> $output_html_file
        <td><a  onclick="toggle('$my_id2')">Show Details</a><br>
	<div id="$my_id2" style='display:none'>$reason</div></td>	
       	</tr> 
End
}

# application_migration_compability()
#
# Get a list of all the depot products on the system.
# For each product, determine whether it can be migrated to HP-9000 Containers:
#
# - Check against a list of known products (known_product_migration.txt)
# - Check for kernel modules (/usr/conf/lib, /usr/conf/mod)
# - Check for programs that use device files (/dev/disk, etc)
# - Check for programs that use unsupported commands (ioscan)
#
# If there are any migration problems, store the details in $tmp_dir/migrate_problems.txt
# Call write_html_application_row to generate the HTML table row for that product.
# The my_idd and my_idd2 are used for generating HTML id attributes, so that
# we can easily show/hide the migration details.

function application_migration_compatibility {
    my_idd=1;
    my_idd2=100000;
    swlist -l product | grep -v "^#" | grep -v "^$" | awk '{print $1}' | \
        grep -v -E "PHCO|PHKL|PHNE|PHSS" > $tmp_dir/product_list.txt

    swlist -l product -l file | grep -v "^#" | grep -v "^$" | \
        grep -v -E "PHCO|PHKL|PHNE|PHSS" > $tmp_dir/product_file_list.txt
    total_product=`cat $tmp_dir/product_list.txt | wc -l`;
    current_product=1;
    cat $tmp_dir/product_list.txt | while read product
    do
        echo "Application: $product ";
        echo "Scanning $current_product of $total_product ";
        echo;
        rm -f $tmp_dir/migrate_problems.txt
        touch $tmp_dir/migrate_problems.txt

        check_if_known_product "$product" "$tmp_dir/migrate_problems.txt"
        status=$?
        if [ $status != 0 ]
        then
            check_if_kernel_module "$product" "$tmp_dir/product_file_list.txt" "$tmp_dir/migrate_problems.txt"
            status=$?
        fi
        if [ $status != 0 ]
        then
            check_product_files "$product" "$tmp_dir/product_file_list.txt" "$tmp_dir/migrate_problems.txt"
        fi

        write_html_application_row "$product" "$tmp_dir/migrate_problems.txt" "$tmp_dir/ldd.out" "$my_idd" "$my_idd2"
        my_idd=`expr $my_idd + 1`;
        my_idd2=`expr $my_idd2 + 1`; 
        current_product=`expr $current_product + 1`;
    done
    echo "Application checking: ... done                   ";
}

function check_user_product {
    product_name=$1
    my_idd=$product_name;
    swlist -l product -l file "$product_name" | grep -v "^#" | grep -v "^$" > $tmp_dir/product_file_list.txt
    echo "Checking Application: $product_name                     ";
    rm -f $tmp_dir/migrate_problems.txt
    touch $tmp_dir/migrate_problems.txt
    check_if_known_product "$product_name" "$tmp_dir/migrate_problems.txt"
    status=$?
    if [ $status != 0 ]; then
        check_if_kernel_module "$product_name" "$tmp_dir/product_file_list.txt" "$tmp_dir/migrate_problems.txt"
        status=$?
    fi
    if [ $status != 0 ]; then
        check_product_files "$product_name" "$tmp_dir/product_file_list.txt" "$tmp_dir/migrate_problems.txt"
    fi
    write_html_application_row "$product_name" "$tmp_dir/migrate_problems.txt" "$tmp_dir/ldd.out" "$my_idd" "$my_idd"
    echo "Application checking: ... done                   ";
}

# check_user_directory()
#
# Gets the non-depot products and files as arguments and scans whether those are migratable or not
# Argument should be a directory and it will scan all the files in the directory
#

function check_user_directory {
    directory=$1
    echo "Checking user directory: $directory                     ";
    my_idd="$directory"
    my_idd2=""
    rm -f $tmp_dir/migrate_problems.txt;
    touch $tmp_dir/migrate_problems.txt;
    rm -f $tmp_dir/ldd.out;
    touch $tmp_dir/ldd.out;
    files=`find $directory -type f`;
    for single_file in `echo $files`; do
        is_pa_risc=`file $single_file | grep "PA-RISC"`
        is_text=`file $single_file | grep "commands text"`
        is_shell_script=""
        if [ "$is_text" != "" ]; then
            is_shell_script=`head -n 1 $single_file | grep "#\!"`
        fi
        if [ "$is_pa_risc" != "" -o "$is_shell_script" != "" ]; then
            does_program_use_device_files $single_file $tmp_dir/migrate_problems.txt
            does_program_use_unsupported_commands $single_file $tmp_dir/migrate_problems.txt
        fi
    done
    write_html_application_row "$directory" "$tmp_dir/migrate_problems.txt" "$tmp_dir/ldd.out" "$directory" "$directory"
}

function detect_oracle {
    rm -f $tmp_dir/migrate_problems.txt;
    touch $tmp_dir/migrate_problems.txt;
    users=`cat /etc/passwd | grep "^ora" | awk -F ":" '{print $1}'`
    for user in `echo $users`; do
        # Check for oracle environment variables
        oracle_paths=`su - $user -c ". .profile; env" 2>/dev/null | grep ORACLE`
        if [ "$oracle_paths" != "" ]
        then
            echo "Oracle is installed in the following path<p>" >> $tmp_dir/migrate_problems.txt
            echo "$oracle_paths" >> $tmp_dir/migrate_problems.txt
            echo "<p>Oracle uses disk device files, so it cannot run under HP-9000 Containers" >> $tmp_dir/migrate_problems.txt
            my_idd="oracle"
            my_idd2="oracle"
            write_html_application_row "Oracle" "$tmp_dir/migrate_problems.txt" "$tmp_dir/ldd.out" "$my_idd" "$my_idd2"
        fi
    done
}

function detect_sap {
    rm -f $tmp_dir/migrate_problems.txt;
    touch $tmp_dir/migrate_problems.txt;
    if [ -d /usr/sap ]
    then
        sap_paths=`find /usr/sap -name sapcontrol`
        if [ "$sap_paths" != "" ]
        then
            echo "SAP is installed in the following path<p>" >> $tmp_dir/migrate_problems.txt
            echo "$sap_paths" >> $tmp_dir/migrate_problems.txt
            my_idd="sap"
            my_idd2="sap"
            write_html_application_row "SAP" "$tmp_dir/migrate_problems.txt" "$tmp_dir/ldd.out" "$my_idd" "$my_idd2"
        fi
    fi
}

########## main block ###############

if [ "$#" != "0" ]; then
    while getopts :ahd:p: OPTION
    do
        case ${OPTION} in
            d|p)
                [[ ${OPTARG} = -* ]]   && print_usage;;
        esac
        case ${OPTION} in
            a) all_applications="TRUE";;
            d) user_directory="$user_directory $OPTARG";;
            p) user_product="$user_product $OPTARG";;
            h) print_usage;;
            :) print_usage;;
            ?) print_usage;;
        esac
    done
    shift $(($OPTIND - 1))
    if [[ -n "$user_product" && -n "$all_applications" ]]; then
        print_usage
    fi
    if [ -n "$*" ]; then
        print_usage
    fi
else
    print_usage
fi

# prevent con-current  multiple instance of this script
if [ -f $lock_file ]; then
    echo "It appears that the script is already running with PID: `cat $lock_file`. " 1>&2
    echo "If this is not the case, remove the lock file $lock_file and re-run this script. " 1>&2
    exit 1;
else
    echo $$ > $lock_file;
fi

check_prereq;

# call cleanup function during exit
trap "cleanup" 0

mkdir -p $tmp_dir;
cp html_report.tmpl $output_html_file;

if [ -n "$user_product" ]; then
    for product_name in $user_product; do
        check_user_product $product_name
    done
fi

if [ -n "$user_directory" ]; then
    for directory_path in $user_directory; do
        if [ $directory_path != "/" ]; then
            if [ -d $directory_path ]; then
                check_user_directory $directory_path
            else
                print_err "The applicaton directory $user_app does not exist";
            fi
        else
            print_err "System root (/) is not allowed. Please select an application directory";
        fi
    done
    if [ -n "$all_applications" ]; then
        if [ -z "$user_product" ]; then
            application_migration_compatibility
            detect_oracle
            detect_sap
        fi
    fi
fi 

if [[ -n "$all_applications" && -z "$user_directory" && -z "$user_product" ]]; then
    application_migration_compatibility
    detect_oracle
    detect_sap
fi    

cat >> $output_html_file <<_HTML_2
       </table><br><p style="width:90%" align="right"><a href="#page_top1">Back To Top</a></p>
  </body>
</html>
_HTML_2

sed -e 's@__date__@'"$start_date"'@' -e 's@__operating_sys__@'"$sys_os"'@' \
   -e 's@__model__@'"$sys_model"'@' -e 's@__server_name__@'"$host_name"'@' \
   -e 's@__argument__@'"$cmd_line"'@' -e 's@__scan9k_version__@'"$scan9k_version"'@'\
   $output_html_file > $tmp_dir/new.html;
mv $tmp_dir/new.html $output_html_file;

create_result;
